# Generated by Django 3.2.13 on 2022-04-28 11:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('drf_operation_log', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='operationlogentry',
            name='domain_content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='contenttypes.contenttype', verbose_name='同范围操作对象'),
        ),
        migrations.AddField(
            model_name='operationlogentry',
            name='domain_object_id',
            field=models.TextField(blank=True, null=True, verbose_name='同范围对象ID'),
        ),
        migrations.AlterField(
            model_name='operationlogentry',
            name='content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='contenttypes.contenttype', verbose_name='操作对象'),
        ),
    ]
